/*   This file is part of Xsltop.
*
*    Xsltop is free software: you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation.
*
*    Xsltop is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY.  See the
*    GNU General Public License for more details.
*
*    See <http://www.gnu.org/licenses/>.
*/
package xsltop.dico;

import java.io.InputStream;
import java.util.HashMap;
import java.util.Map;

import org.w3c.dom.Node;

import xsltop.xml.XmlBaseEngine;
import xsltop.xml.XsltopException;
/**
 * Read and set in memory the lang.dico files.
 * <p>This class is used to translate the GUI labels.</p>
 * <p>This class is a singleton used in pool of dico.</p>
 * @author philippe voncken
 * @see xsltop.dico.Labels
 *
 */
public class Dico {
	/** the DOM to keep data in memory. */
	private XmlBaseEngine doc = null;
	/** The pool of dictionnary. */
	private static Map<String, Dico> singleton = new HashMap<String, Dico>();
	/** No default constructor. */
	private Dico() {		
	}
	/**
	 * Read the dico file.
	 * @param lang the dico file in the classpath.
	 */
	private Dico(String lang) {
		InputStream file = Dico.class.getResourceAsStream("/" + lang);
		try {
			doc = new XmlBaseEngine(file, false, false);
		} catch (XsltopException exception) {
			doc = null;
		}
	}
	/**
	 * the getter to the pool.
	 * @param lang the dico which we want
	 * @return the dico which is used to translate the GUI labels
	 */
	public synchronized static Dico getInstance(String lang) {
		Dico dico = singleton.get(lang);
        if (dico == null) {
        	dico = new Dico(lang);
        	singleton.put(lang, dico);
        }
        return dico;
    }
	/**
	 * the label getter.
	 * @param xpath expression which contains in Labels interface
	 * @return a String label
	 * @see xsltop.dico.Labels
	 */
	public String getLabel(String xpath) {
		String item;
		try {
			Node node = doc.evaluateXPathSingle(xpath);
			if (node != null) {
				item = node.getNodeValue();
			} else {
				System.out.println("Error on dico, xpath: " + xpath);
				item = new String("---");
			}			 
		} catch (XsltopException exception) {
			item = new String("---");
		}
		return item;
	}
}
