/*   This file is part of Xsltop.
*
*    Xsltop is free software: you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation.
*
*    Xsltop is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY.  See the
*    GNU General Public License for more details.
*
*    See <http://www.gnu.org/licenses/>.
*/
package xsltop.util;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
/**
 * help to the use of stream.
 * @author philippe voncken
 *
 */
public class StreamHelper {
	/**
	 * Transform a file to a String.
	 * @param stream to transform
	 * @return a String
	 * @throws IOException if IO error
	 */
	public static String fileToString(InputStream stream) throws IOException {
		StringBuilder string = new StringBuilder();
		BufferedReader reader = inputStreamToReader(stream);
		int caractere = reader.read(); 
		while (caractere != -1) {			
			string.append((char) caractere);
			caractere = reader.read();
		}
		reader.close();
		return string.toString();
	}
	private static BufferedReader inputStreamToReader(InputStream inputStream) {
		return new BufferedReader(
				new InputStreamReader(inputStream)
				);
	}
	/**
	 * Dico Filter on the classpath.
	 * @param list of String which contains the files name
	 * @return list of String which contains the name of the dico file
	 */
	public static List<String> selectDicoFile(List<String> list) {
		List<String> save = new LinkedList<String>();
		Iterator<String> iterator = list.iterator();
		while (iterator.hasNext()) {
			final String element = iterator.next();
			if (element.endsWith(".dico")) {
				save.add(element);
			}
		}		
		return save;
	}
	/**
	 * Find namespace in a String which contains a XML.
	 * @param xml String which contains a XML
	 * @return a Map of the namespace and prefix found 
	 */
	public static Map<String, String> findNamespace(String xml) {
		Map<String, String> map = new HashMap<String, String>();
		int indexNS = xml.indexOf(" xmlns=");
        if (indexNS != -1) {
            int indexEqual = indexNS + 7;
            char apostrophe = xml.charAt(indexEqual);
            String prefix = "ns";
            int indexEndUri = xml.indexOf(apostrophe, indexEqual + 1);                
            String uri = xml.substring(indexEqual + 1, indexEndUri);
            map.put(prefix, uri);
        }
		int index = xml.indexOf(" xmlns:");
		while (index != -1) {
			int indexEqual = xml.indexOf("=", index);
			if (indexEqual != -1) {
				char apostrophe = xml.charAt(indexEqual + 1);
				String prefix = xml.substring(index + 7, indexEqual);
				int indexEndUri = xml.indexOf(apostrophe, indexEqual + 2);				
				String uri = xml.substring(indexEqual + 2, indexEndUri);
				map.put(prefix, uri);
				index = xml.indexOf(" xmlns:", index + 1);
			} else {
				index = -1;
			}
		}
		return map;
	}	
}
