/*   This file is part of Xsltop.
*
*    Xsltop is free software: you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation.
*
*    Xsltop is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY.  See the
*    GNU General Public License for more details.
*
*    See <http://www.gnu.org/licenses/>.
*/
package xsltop.xml;

import java.io.File;
import java.io.IOException;
import java.io.StringWriter;

import javax.xml.transform.OutputKeys;
import javax.xml.transform.Result;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMResult;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

import org.w3c.dom.Document;

import com.sun.org.apache.xml.internal.serialize.OutputFormat;
import com.sun.org.apache.xml.internal.serialize.XMLSerializer;

/**
 * The Class XsltopTransformer.
 */
public class XsltopTransformer {
	
	/** The Constant YES. */
	private static final String YES = "yes";
	
	/** The Constant UTF8. */
	private static final String UTF8 = "utf-8";
	
	/** The Constant XML. */
	private static final String XML = "xml";
	
	/**
	 * Instantiates a new xsltop transformer.
	 */
	private XsltopTransformer() {
		super();
	}	
    
    /**
     * Xml to string.
     * 
     * @param pDoc the document
     * 
     * @return the string
     * 
     * @throws XsltopException the xsltop exception
     */
    public static String exportToString(final Document pDoc)
    throws XsltopException {
        String vReturn = null;
        if (null == pDoc) {
            throw new XsltopException(
                    "XmlTransformer.xmlToString(final Document document) "
                    + "-> Pas de document à transformer");
        }        
        final StringWriter writer = new StringWriter();
        OutputFormat format = new OutputFormat(pDoc);
        format.setIndenting(true);
        format.setIndent(4);
        XMLSerializer serializer = new XMLSerializer(writer, format);
        try {
            serializer.serialize(pDoc);
            vReturn = writer.toString();
        } catch (IOException e) {
            System.out.println(e);
        }
        return vReturn; 
    }
	
	/**
	 * Xml to file.
	 * 
	 * @param pDoc the document
	 * @param pFile the file
	 * 
	 * @throws XsltopException the xsltop exception
	 */
	public static void exportToFile(final Document pDoc, final File pFile)
	throws XsltopException {
		if ((null == pDoc) || (null == pFile)) {
			throw new XsltopException("Pas de document à transformer");
		}		
		final DOMSource domSource = new DOMSource(pDoc);
        final Result result = new StreamResult(pFile.getAbsolutePath());                
        try {
        	final Transformer transformer = getTransformer(pDoc);
            transformer.transform(domSource, result);
        } catch (TransformerException e) {
        	throw new XsltopException(e);
        } 
	}
	
	/**
	 * Gets the transformer.
	 * 
	 * @param pDoc the document
	 * 
	 * @return the transformer
	 * 
	 * @throws TransformerException the transformer exception
	 */
	private static Transformer getTransformer(final Document pDoc)
	throws TransformerException {
		final Transformer transformer = TransformerFactory.newInstance()
		.newTransformer();
		transformer.setOutputProperty(OutputKeys.INDENT, YES);
		transformer.setOutputProperty(OutputKeys.ENCODING, UTF8);
		transformer.setOutputProperty(OutputKeys.METHOD, XML);
		if (pDoc.getDoctype() != null) {
			transformer.setOutputProperty(OutputKeys.DOCTYPE_PUBLIC, 
				pDoc.getDoctype().getPublicId());
			transformer.setOutputProperty(OutputKeys.DOCTYPE_SYSTEM, 
				pDoc.getDoctype().getSystemId());
		}
		return transformer;
	}
	
	/**
	 * Transform.
	 * 
	 * @param pXml the xml
	 * @param pXsl the xsl
	 * 
	 * @return the string
	 */
	public static String transform(final Document pXml, final Document pXsl) {
        String vReturn;
        try {
            final Transformer transformer = 
                TransformerFactory.newInstance().newTransformer(
                new DOMSource(pXsl)
                );
            transformer.setOutputProperty(OutputKeys.INDENT, YES);
            transformer.setOutputProperty(OutputKeys.ENCODING, UTF8);
            transformer.setOutputProperty(OutputKeys.METHOD, XML);
          
            final StringWriter writer = new StringWriter();
            final DOMResult result = new DOMResult();
                                    
            transformer.transform(new DOMSource(pXml), result);
            
            Document docResult = (Document)result.getNode();
            
            OutputFormat format = new OutputFormat(docResult);
            format.setIndenting(true);
            format.setIndent(4);
            format.setMethod(XML);
            format.setEncoding(UTF8);
            XMLSerializer serializer = new XMLSerializer(writer, format);
            serializer.serialize(docResult);
            vReturn = writer.toString();
            
            vReturn = writer.toString();
        } catch (Exception e) {
            System.out.println(e.getMessage());
            vReturn = e.getMessage();
        }
        return vReturn;
    }
}
